'use client'
import { useEffect, useRef, useState } from "react";
import useMessagesStore from "@/feature/messenger/store/chat_store/useMessagesStore";
import useConfigChatStore from "@/feature/messenger/store/chat_store/useConfigChatStore";
import useConversationsStore from "@/feature/messenger/store/chat_store/useConversationsStore";
import RepliesMessage from "./RepliesMessage";
import SentMessage from "./SentMessage";
import { Tooltip } from "@nextui-org/react";
import { useTranslation } from "@/src/hooks/useTranslation";
import { useAuthStore } from "@/lib/auth/hooks/auth-store";


const Messages = () => {
    
    //state
    const id_chatbot = useConfigChatStore(state => state.idChatbot);
    const selectedConversation = useConversationsStore(state => state.selectedConversation);
    const sesionUser = selectedConversation.sesion == '' ? "" : selectedConversation.sesion;
    const listMessages = useMessagesStore(state => state.listMessages);
    const actionsMessages = useMessagesStore(state => state.actions);
    const { trans } = useTranslation();
    const workerRef = useRef<Worker | null>(null);
    const [authToken, setAuthToken] = useState<string | null>(null);

    // Get auth token
    useEffect(() => {
        const getToken = async () => {
            const token = await useAuthStore.getState().getAuthToken();
            setAuthToken(token);
        };
        getToken();
    }, []);

    const handleSeeMore = () => {
        const session = selectedConversation.sesion;
        if (session !== '') {
            actionsMessages.consultHistory(id_chatbot, session);
        }
    }

    // Initialize the Web Worker
    useEffect(() => {
        // Create the worker only on the client side
        if (typeof window !== 'undefined') {
            // Initialize the worker
            workerRef.current = new Worker(
                new URL('./messagesWorker.ts', import.meta.url)
            );

            // Set up the event listener for messages from the worker
            workerRef.current.addEventListener('message', (event) => {
                const { type, payload } = event.data;
                
                if (type === 'MESSAGES_FETCHED') {
                    if (payload.data && payload.data.length > 0) {
                        actionsMessages.setListMessages(payload.data);
                    } else {
                        actionsMessages.setListMessages([]);
                    }
                    if (payload.loading === true) {
                        actionsMessages.setLoading(false);
                    }
                } else if (type === 'ERROR') {
                    console.error('Worker error:', payload);
                    actionsMessages.setLoading(false);
                }
            });
        }

        // Cleanup function to terminate the worker when component unmounts
        return () => {
            if (workerRef.current) {
                workerRef.current.postMessage({ type: 'STOP_POLLING' });
                workerRef.current.terminate();
                workerRef.current = null;
            }
        };
    }, []);

    // Handle message polling with the worker
    useEffect(() => {
        if (!workerRef.current || sesionUser === "") return;

        const apiUrl = process.env.BASE_URL_API;

        if (process.env.NEXT_CHAT_ENABLE_POLLING === "true") {
            if (sesionUser !== "") {
                // Stop any existing polling
                workerRef.current.postMessage({ type: 'STOP_POLLING' });
                
                // Start polling with the worker
                workerRef.current.postMessage({
                    type: 'START_POLLING',
                    payload: {
                        id_chatbot,
                        session: sesionUser,
                        pollingInterval: parseInt(process.env.NEXT_CHAT_TIME_POLLING || "15000"),
                        apiUrl,
                        token: authToken
                    }
                });
            }
        } else {
            if (sesionUser !== "") {
                actionsMessages.setListMessages([]);
                actionsMessages.setLoading(true);
                
                // Fetch messages once with the worker
                workerRef.current.postMessage({
                    type: 'FETCH_ONCE',
                    payload: {
                        id_chatbot,
                        session: sesionUser,
                        apiUrl,
                        loading: true,
                        token: authToken
                    }
                });
            }
        }
    }, [selectedConversation, id_chatbot, sesionUser, authToken]);

    return (
        <div className="w-full h-full mx-auto">
            {/*Messages*/}
            <div className="w-full h-full p-6 overflow-y-auto overflow-x-hidden bg-wallpaper_chat_two dark:bg-gray-700 transition-all duration-150 scrollbar" id="chating-messages">
                {/*Ver mas */}
                <div className={`flex flex-row justify-center items-center ${listMessages.length > 0 ? "" : "hidden"}`}>
                    <Tooltip
                        placement="bottom"
                        content={trans('content_chat_show_more_button_title')}
                        color="default"
                    >
                        <span className="bg-blue-100 text-black text-sm font-medium me-2 px-2.5 py-0.5 rounded-full dark:bg-blue-900 dark:text-blue-300 cursor-pointer" onClick={handleSeeMore}>Ver Mas</span>
                    </Tooltip>
                </div>
                {listMessages.map((item, index) => {
                    return (
                        <>
                            {item.quien_ejecuta_proceso === 'usuario' ? <SentMessage key={index} item={item} /> : <RepliesMessage key={index} item={item} />}
                        </>
                    )
                })}
            </div>
        </div>
    )
}

export default Messages